﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Forms;
using System.Windows.Input;
using System.Windows.Markup;
using System.Windows.Media;
using System.Windows.Xps.Packaging;
using JeffreySun.Dwfx.Converting;

namespace JeffreySun.Dwfx2XamlConverter
{
  /// <summary>
    /// Interaction logic for ContentView.xaml
  /// </summary>
  public partial class ContentView : Window
  {
    private XpsDocument xpsDocument;
    private string fileName;
    private static RoutedCommand ConvertCommand;

    private static MouseButtonEventHandler ScallbarHanlder;
    private static RoutedEventHandler ObjectMouseClickHandler;

    public ContentView()
    {
      InitializeComponent();

      ContentView.ScallbarHanlder = this.RestoreScalingFactor;
      ContentView.ObjectMouseClickHandler = this.OnMouseDown;

      CommandBindings.Add(new CommandBinding(ApplicationCommands.Open, OpenCommandHandler));
      CommandBindings.Add(new CommandBinding(ApplicationCommands.Close, CloseCommandHandler));
      CommandBindings.Add(new CommandBinding(ApplicationCommands.Help, AboutCommandHandler));

      ConvertCommand = new RoutedCommand("Convert", typeof(ContentView));
      ConvertMenuItem.Command = ConvertCommand;

      CommandBindings.Add(new CommandBinding(ConvertCommand, ConvertCommandHandler));
    }

    private void AboutCommandHandler(object sender, ExecutedRoutedEventArgs e)
    {
      AboutInformation about = new AboutInformation();
      about.ShowDialog();
    }

    /// <summary>
    ///   Opens an existing XPS document and displays it with a DocumentViewer.
    ///   </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void OpenCommandHandler(object sender, ExecutedRoutedEventArgs e)
    {
        this.OpenDwfxFile();
    }

    /// <summary>
    /// File|Close handler - Closes current XPS document.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void CloseCommandHandler(object sender, ExecutedRoutedEventArgs e)
    {
      if (null != this.xpsDocument)
      {
        this.xpsDocument.Close();
      }

      this.xpsDocument = null;
      this.Close();
    }

    /// <summary>
    /// File|Convert handler
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ConvertCommandHandler(object sender, ExecutedRoutedEventArgs e)
    {
        this.ConvertAndPreview();
    }

    private void openButton_Click(object sender, RoutedEventArgs e)
    {
        this.OpenDwfxFile();
    }

    private void btnSave_Click(object sender, RoutedEventArgs e)
    {

    }

    private void btnExit_Click(object sender, RoutedEventArgs e)
    {
      this.Close();
    }

    private void btnConvert_Click(object sender, RoutedEventArgs e)
    {
        this.ConvertAndPreview();
    }

    private void btnOption_Click(object sender, RoutedEventArgs e)
    {

    }

    private void btnZoomIn_Click(object sender, RoutedEventArgs e)
    {
        double ratio = this.scaleSlider.Value + 2;
        this.scaleSlider.Value = ratio <= this.scaleSlider.Maximum ? ratio : this.scaleSlider.Maximum;
    }

    private void btnZoomOut_Click(object sender, RoutedEventArgs e)
    {
        double ratio = this.scaleSlider.Value - 2;
        this.scaleSlider.Value = ratio >= this.scaleSlider.Minimum ? ratio : this.scaleSlider.Minimum;
    }

    private void btnAbout_Click(object sender, RoutedEventArgs e)
    {
      AboutInformation about = new AboutInformation();
      about.ShowDialog();
    }

    private void OpenDwfxFile()
    {
        OpenFileDialog dlg = new OpenFileDialog();
        dlg.Filter = "DWFX Drawings (*.dwfx)|*.dwfx";
        dlg.FilterIndex = 1;
        if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
        {
            if (xpsDocument != null)
            {
                xpsDocument.Close();
            }
            try
            {
                xpsDocument = new XpsDocument(dlg.FileName, System.IO.FileAccess.Read);
            }
            catch (UnauthorizedAccessException)
            {
                System.Windows.MessageBox.Show(
                    String.Format("Unable to access {0}", dlg.FileName));
                return;
            }

            docViewer.Document = xpsDocument.GetFixedDocumentSequence();
            fileName = dlg.FileName;

            this.contentViewer.SelectedIndex = 0;

            if (null != this.convertedView.Child)
            {
                this.scaleSlider.MouseDoubleClick -= ContentView.ScallbarHanlder;
                this.convertedView.Child.RemoveHandler(System.Windows.Shapes.Path.MouseDownEvent, ContentView.ObjectMouseClickHandler);
            }
        }
    }

    private void ConvertAndPreview()
    {
        SaveFileDialog dialog = new SaveFileDialog();
        dialog.Filter = "WPF Files (*.xaml)|*.xaml";
        dialog.InitialDirectory = Directory.GetCurrentDirectory();

        if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
        {
            Converter converter = new Converter(this.fileName);
            converter.ConvertTo(dialog.FileName);

            Stream stream = File.Open(dialog.FileName, FileMode.Open, FileAccess.Read);
            UIElement xamlView = XamlReader.Load(stream) as UIElement;
            this.convertedView.Child = xamlView;

            this.scaleSlider.MouseDoubleClick += ContentView.ScallbarHanlder;
            this.convertedView.Child.AddHandler(System.Windows.Shapes.Path.MouseDownEvent, ContentView.ObjectMouseClickHandler);

            this.contentViewer.SelectedIndex = 1;
        }
    }

    private void RestoreScalingFactor(object sender, MouseButtonEventArgs args)
    {
        ((Slider)sender).Value = 1.0;
    }

    private void OnPreviewMouseWheel(object sender, MouseWheelEventArgs e)
    {
        base.OnPreviewMouseWheel(e);

        if (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl))
        {
            scaleSlider.Value += (e.Delta > 0) ? 1 : -1;
        }
    }

    private void OnMouseDown(object sender, RoutedEventArgs e)
    {
        System.Windows.Shapes.Path line = e.OriginalSource as System.Windows.Shapes.Path;
        if (null == line)
        {
            return;
        }

        line.Fill = new SolidColorBrush(Colors.Yellow);
        this.txtBlockInfo.Text = line.Tag.ToString();
    }
  }
}
