﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Packaging;
using System.Windows.Documents;
using System.IO;
using System.Windows.Markup;

namespace JeffreySun.Dwfx.Packing
{
  public class DwfxDocument : DwfxPartBase, INode, IDisposable
  {
    private bool isDisposed;
    private bool isInDocumentStage;
    private Package dwfxPackage;
    private DwfxThumbnail thumbnail;

    public DwfxDocument(Package package)
      : this(package, CompressionOption.Maximum)
    {
    }

    public DwfxDocument(Package package, CompressionOption compressionOption)
      : this(package, compressionOption, null)
    {
    }

    public DwfxDocument(string path, FileAccess packageAccess)
      : this(path, packageAccess, CompressionOption.Maximum)
    {
    }

    public DwfxDocument(Package package, CompressionOption compressionOption, string path) :
      base(new DwfxManager(package, compressionOption))
    {
      if (path != null)
      {
        base.Uri = new Uri(path, UriKind.RelativeOrAbsolute);
      }
      base.CurrentDwfxManager.DwfxDocument = this;
      this.Initialize();
    }

    public DwfxDocument(string path, FileAccess packageAccess, CompressionOption compressionOption)
      : base(new DwfxManager(path, packageAccess, compressionOption))
    {
      if (path == null)
      {
        throw new ArgumentNullException();
      }

      base.Uri = new Uri(path, UriKind.RelativeOrAbsolute);
      if (!base.Uri.IsAbsoluteUri)
      {
        base.Uri = new Uri(new Uri(Directory.GetCurrentDirectory() + "/"), base.Uri);
      }

      base.CurrentDwfxManager.DwfxDocument = this;
      this.Initialize();
    }

    public FixedDocumentSequence GetFixedDocumentSequence()
    {
      this.CheckDisposed();
      if (base.CurrentDwfxManager == null)
      {
        throw new InvalidOperationException(SR.Get("ReachPackaging_DocumentWasClosed"));
      }
      if (null == base.Uri)
      {
        throw new DwfxPackagingException(SR.Get("ReachPackaging_PackageUriNull"));
      }
      if (base.CurrentDwfxManager.StartingPart == null)
      {
        return null;
      }

      ParserContext parserContext = new ParserContext();
      parserContext.BaseUri = PackUriHelper.Create(base.Uri, base.CurrentDwfxManager.StartingPart.Uri);
      object obj2 = XamlReader.Load(base.CurrentDwfxManager.StartingPart.GetStream(), parserContext);
      if (!(obj2 is FixedDocumentSequence))
      {
        throw new DwfxPackagingException(SR.Get("ReachPackaging_NotAFixedDocumentSequence"));
      }

      return (obj2 as FixedDocumentSequence);
    }

    public PackageProperties CoreDocumentProperties
    {
      get
      {
        this.CheckDisposed();
        return base.CurrentDwfxManager.MetroPackage.PackageProperties;
      }
    }

    public IDwfxDocumentSequenceReader DwfxDocumentSequenceReader
    {
      get
      {
        this.CheckDisposed();
        DwfxDocumentSequenceReader reader = null;
        PackagePart startingPart = base.CurrentDwfxManager.StartingPart;
        if (startingPart != null)
        {
          reader = new DwfxDocumentSequenceReader(base.CurrentDwfxManager, null, startingPart);
        }

        return reader;
      }
    }

    public PackagePart GetPart()
    {
      return null;
    }

    public void Dispose()
    {
      this.Dispose(true);
      GC.SuppressFinalize(this);
    }

    protected virtual void Dispose(bool disposing)
    {
      if (!this.isDisposed)
      {
        if (disposing)
        {
          this.thumbnail = null;
          this.dwfxPackage = null;
          base.CurrentDwfxManager.Dispose();
        }

        this.isDisposed = true;
      }
    }

    private void Initialize()
    {
      this.isInDocumentStage = false;
      this.dwfxPackage = null;
    }

    private void CheckDisposed()
    {
      if (this.isDisposed)
      {
        throw new ObjectDisposedException("DwfxDocument");
      }
    }

    private void EnsureThumbnail()
    {
      if (this.thumbnail == null)
      {
        this.thumbnail = base.CurrentDwfxManager.EnsureThumbnail(this, null);
      }
    }
  }
}
