﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Packaging;
using System.Collections.ObjectModel;
using System.IO;
using System.Xml;

namespace JeffreySun.Dwfx.Packing
{
  public class DwfxDocumentReader : DwfxPartBase, IDwfxDocumentReader, INode, IDisposable
  {
    private IDwfxFixedPageReader currentFixedPage;
    private IDwfxObjectDefinitionReader objectDefinition;
    private IDwfxGraphicsExtentionReader graphicsExtension;
    private IDwfxGraphicsDictionaryReader graphicsDictionary;

    private Dictionary<string, List<DwfxProperty>> blockPropertiesDictionary;
    private List<DwfxFont> fontCollection;

    private int documentNumber;
    private DwfxStructure documentStructure;
    private bool hasParsedPages;
    private IList<string> linkTargetStream;
    private PackagePart metroPart;
    private List<IDwfxContentReader> pageCache;
    private INode parentNode;
    private DwfxThumbnail thumbnail;
    private DwfxStructure xpsStructure;

    public DwfxDocumentReader(DwfxManager dwfxManager, INode parent, PackagePart part, int documentNumber) : base(dwfxManager)
    {
        if (part == null)
        {
            throw new ArgumentNullException("part");
        }

        base.Uri = part.Uri;
        this.metroPart = part;
        this.pageCache = new List<IDwfxContentReader>();
        this.fontCollection = new List<DwfxFont>();
        this.parentNode = parent;
        this.hasParsedPages = false;
        this.documentNumber = documentNumber;
    }

    public IDwfxFixedPageReader CurrentFixedPage
    {
      get
      {
        return this.currentFixedPage;
      }
    }

    public IDwfxObjectDefinitionReader DefinitionReader
    {
      get
      {
        return this.objectDefinition;
      }
    }

    public IDwfxGraphicsExtentionReader ExtensionReader
    {
      get
      {
        return this.graphicsExtension;
      }
    }

    public IDwfxGraphicsDictionaryReader DictionaryReader
    {
      get
      {
        return this.graphicsDictionary;
      }
    }

    public List<DwfxFont> FontCollection
    {
        get
        {
            return this.fontCollection;
        }
    }

    public PackagePart MetroPart
    {
      get
      {
        return this.metroPart;
      }
    }

    public IDwfxDocumentReader ParentNode
    {
      get
      {
        return null;
      }
    }

    public Dictionary<string, List<DwfxProperty>> BlockPropertiesDictionary
    {
      get
      {
        if (null == this.blockPropertiesDictionary)
        {
          this.blockPropertiesDictionary = new Dictionary<string, List<DwfxProperty>>(ExtensionReader.ExtensionDictionary.Count);
          foreach (KeyValuePair<string, DwfxExtensionNode> extension in ExtensionReader.ExtensionDictionary)
          {
            if (objectDefinition.InstancesDictionary.ContainsKey(extension.Value.ObjectNodeNumber))
            {
              DwfxInstance instance = objectDefinition.InstancesDictionary[extension.Value.ObjectNodeNumber];
              if (objectDefinition.ObjectsDictionary.ContainsKey(instance.ObjectId))
              {
                DwfxObject obj = objectDefinition.ObjectsDictionary[instance.ObjectId];
                if (objectDefinition.PropertiesDictionary.ContainsKey(obj.propertyGroupRefId))
                {
                  this.blockPropertiesDictionary.Add(extension.Key, objectDefinition.PropertiesDictionary[obj.propertyGroupRefId]);
                }
              }
            }
          }
        }

        return this.blockPropertiesDictionary;
      }
    }

    public IDwfxFixedPageReader GetFixedPage(Uri pageUri)
    {
      this.UpdatePageCache();
      IDwfxFixedPageReader reader = null;
      foreach (IDwfxContentReader cachedReader in this.pageCache)
      {
        if (cachedReader.Uri == pageUri && (cachedReader is IDwfxFixedPageReader))
        {
          reader = cachedReader as IDwfxFixedPageReader;
        }
      }

      return reader;
    }

    public IDwfxObjectDefinitionReader GetObjectDefinition(Uri pageUri)
    {
      this.UpdatePageCache();
      IDwfxObjectDefinitionReader reader = null;
      foreach (IDwfxContentReader cachedReader in this.pageCache)
      {
        if (cachedReader.Uri == pageUri && (cachedReader is IDwfxObjectDefinitionReader))
        {
          reader = cachedReader as IDwfxObjectDefinitionReader;
        }
      }

      return reader;
    }

    public int DocumentNumber
    {
      get
      {
        return this.documentNumber;
      }
    }

    public DwfxStructure DocumentStructure
    {
      get
      {
        this.EnsureDoucmentStructure();
        return this.documentStructure;
      }
    }

    public ReadOnlyCollection<IDwfxFixedPageReader> FixedPages
    {
      get
      {
        this.UpdatePageCache();

        List<IDwfxFixedPageReader> fixedPageReader = new List<IDwfxFixedPageReader>();
        foreach (IDwfxContentReader part in this.pageCache)
        {
          if (part is IDwfxFixedPageReader)
          {
            fixedPageReader.Add(part as IDwfxFixedPageReader);
          }
        }
        return new ReadOnlyCollection<IDwfxFixedPageReader>(fixedPageReader);
      }
    }

    public PackagePart GetPart()
    {
        return this.metroPart;
    }

    public void Dispose()
    {
      this.metroPart = null;
      this.parentNode = null;
      this.thumbnail = null;
      this.pageCache = null;
      this.hasParsedPages = false;
    }

    private IDwfxContentReader AddPageToCache(Uri pageUri, string role)
    {
      PackagePart part = base.CurrentDwfxManager.GetPart(pageUri);
      if (part == null)
      {
        throw new DwfxPackagingException(SR.Get("ReachPackaging_PartNotFound"));
      }

      IDwfxContentReader item = null;;
      switch (role)
      {
        case DwfxContentRoles.ObjectDefination:
          item = this.objectDefinition = new DwfxObjectDefinitionReader(base.CurrentDwfxManager, this, part);
          break;
        case DwfxContentRoles.GraphicsDictionary:
          item = this.graphicsDictionary = new DwfxGraphicsDictionaryReader(base.CurrentDwfxManager, this, part);
          break;
        case DwfxContentRoles.StreamingGraphics:
          item = this.currentFixedPage = new DwfxFixedPageReader(base.CurrentDwfxManager, this, part);
          break;
        case DwfxContentRoles.GraphicsExtension:
          item = this.graphicsExtension = new DwfxGraphicsExtensionReader(base.CurrentDwfxManager, this, part);
          break;
         case DwfxContentRoles.Font:
          DwfxFont font = new DwfxFont(base.CurrentDwfxManager, this, part);
          this.fontCollection.Add(font);
          break;
        default:
          break;
      }

      this.pageCache.Add(item);
      return item;
    }

    private void EnsureDoucmentStructure()
    {
      if (this.documentStructure == null)
      {
        PackageRelationship relationship = null;
        PackagePart part = null;
        foreach (PackageRelationship relationship2 in this.metroPart.GetRelationshipsByType(DwfxMarkup.StructureRelationshipName))
        {
          if (relationship != null)
          {
            throw new InvalidDataException(SR.Get("ReachPackaging_MoreThanOneDocStructure"));
          }
          relationship = relationship2;
        }

        if (relationship != null)
        {
          Uri partUri = PackUriHelper.ResolvePartUri(relationship.SourceUri, relationship.TargetUri);
          if (base.CurrentDwfxManager.MetroPackage.PartExists(partUri))
          {
            part = base.CurrentDwfxManager.MetroPackage.GetPart(partUri);
            this.documentStructure = new DwfxStructure(base.CurrentDwfxManager, this, part);
          }
        }
      }
    }

    public void ResolveContent()
    {
      this.UpdatePageCache();

      foreach (IDwfxContentReader contentReader in this.pageCache)
      {
        if (contentReader != null)
        {
          contentReader.ResolveContent();
        }
      }
    }

    public DwfxThumbnail Thumbnail
    {
      get
      {
        this.EnsureThumbnail();
        return this.thumbnail;
      }
    }

    private void EnsureThumbnail()
    {
        if (this.thumbnail == null)
        {
            this.thumbnail = base.CurrentDwfxManager.EnsureThumbnail(this, this.metroPart);
        }
    }

    private void UpdatePageCache()
    {
      if (!this.hasParsedPages)
      {
        this.ParsePages();
        this.hasParsedPages = true;
      }
    }

    private void ParsePages()
    {
      Stream input = this.metroPart.GetStream(FileMode.Open);
      if (input.Length > 0L)
      {
        using (XmlTextReader reader = new XmlTextReader(input))
        {
          while (reader.Read())
          {
            string s = reader.GetAttribute(XmlTags.Type);
            Console.WriteLine(s);
            if (s == DwfxMarkup.DwfxContentSectionType)
            {
              Console.WriteLine(s);
            }

            if ((reader.NodeType == XmlNodeType.Element) && (reader.LocalName == XmlTags.Section) &&
              (reader.GetAttribute(XmlTags.Type) == DwfxMarkup.DwfxContentSectionType))
            {
              using (XmlReader contentReader = reader.ReadSubtree())
              {
                while (contentReader.Read())
                {
                  if ((contentReader.NodeType == XmlNodeType.Element) && (contentReader.LocalName == XmlTags.Resource))
                  {
                    string role = contentReader.GetAttribute(XmlTags.Role);
                    string link = contentReader.GetAttribute(XmlTags.Link);

                    if (role == DwfxContentRoles.StreamingGraphics)
                    {
                      link.Substring(0, link.IndexOf('?'));
                    }

                    Uri targetUri = new Uri(link, UriKind.Relative);
                    this.AddPageToCache(PackUriHelper.ResolvePartUri(base.Uri, targetUri), role);
                  }
                }
              }

              break;
            }
          }
        }
      }
    }

  }
}
