﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.IO.Packaging;
using System.Net.Mime;

namespace JeffreySun.Dwfx.Packing
{
  public class DwfxManager : IDisposable
  {
    private Dictionary<Uri, PackagePart> cachedParts;
    private CompressionOption compressionOption;
    private Dictionary<string, int> contentTypesDictionary;
    internal static object globalLock = new object();
    private Package metroPackage;
    private bool isOwnsPackage;
    internal static Dictionary<Uri, int> packageCache = new Dictionary<Uri, int>();
    private Uri uri;
    private DwfxDocument dwfxDocument;

    public DwfxManager(Package mPackage, CompressionOption compressionOption)
    {
      this.isOwnsPackage = false;

      this.Initialize(metroPackage, compressionOption);
    }

    public DwfxManager(string path, FileAccess packageAccess, CompressionOption compressionOption)
    {
      this.isOwnsPackage = true;
      this.uri = new Uri(path, UriKind.RelativeOrAbsolute);
      if (!this.uri.IsAbsoluteUri)
      {
        this.uri = new Uri(new Uri(Directory.GetCurrentDirectory() + "/"), path);
      }

      Package package = PackageStore.GetPackage(this.uri);
      if (package == null)
      {
        if (packageAccess == FileAccess.Write)
        {
            throw new InvalidOperationException("This library just supports readonly using");
        }
        else
        {
          package = Package.Open(path, (packageAccess == FileAccess.Read) ? FileMode.Open : FileMode.OpenOrCreate, packageAccess, (packageAccess == FileAccess.Read) ? FileShare.Read : FileShare.None);
        }

        this.AddPackageToCache(this.uri, package);
      }
      else
      {
        if ((packageAccess != FileAccess.Read) || (package.FileOpenAccess != FileAccess.Read))
        {
          throw new UnauthorizedAccessException();
        }

        this.AddPackageReference(this.uri);
      }

      this.Initialize(package, compressionOption);
    }

    public Package MetroPackage
    {
      get
      {
        return this.metroPackage;
      }
    }

    public static string MakeRelativePath(Uri baseUri, Uri fileUri)
    {
      Uri uri = new Uri("http://dummy");
      if (!baseUri.IsAbsoluteUri)
      {
        baseUri = new Uri(uri, baseUri);
      }
      if (!fileUri.IsAbsoluteUri)
      {
        fileUri = new Uri(uri, fileUri);
      }
      Uri uri3 = new Uri(baseUri.MakeRelativeUri(fileUri).GetComponents(UriComponents.SerializationInfoString, UriFormat.SafeUnescaped), UriKind.RelativeOrAbsolute);
      return uri3.GetComponents(UriComponents.SerializationInfoString, UriFormat.UriEscaped);
    }

    private void Initialize(Package metroPackage, CompressionOption compressionOption)
    {
      if (metroPackage == null)
      {
        throw new ArgumentNullException("Package");
      }

      this.dwfxDocument = null;
      this.metroPackage = metroPackage;
      this.compressionOption = compressionOption;
      this.contentTypesDictionary = new Dictionary<string, int>(11);
      this.cachedParts = new Dictionary<Uri, PackagePart>(11);
    }

    private void AddPackageReference(Uri uri)
    {
      lock (globalLock)
      {
        if (!packageCache.ContainsKey(uri))
        {
          packageCache.Add(uri, 0);
        }

        packageCache[uri] += 1;
      }
    }

    private void AddPackageToCache(Uri uri, Package package)
    {
      lock (globalLock)
      {
        packageCache[uri] = 1;
      }

      PackageStore.AddPackage(uri, package);
    }

    public PackagePart GetPart(Uri uri)
    {
      if (this.metroPackage == null)
      {
        throw new ObjectDisposedException("DwfxManager");
      }
      if (this.cachedParts.ContainsKey(uri))
      {
        return this.cachedParts[uri];
      }
      if (!this.metroPackage.PartExists(uri))
      {
        return null;
      }

      PackagePart part = this.metroPackage.GetPart(uri);
      this.cachedParts[uri] = part;
      return part;
    }

    public DwfxThumbnail EnsureThumbnail(INode parent, PackagePart part)
    {
        DwfxThumbnail thumbnail = null;
        PackagePart thumbnailPart = this.GetThumbnailPart(part);
        if (thumbnailPart != null)
        {
            thumbnail = new DwfxThumbnail(this, parent, thumbnailPart);
        }
        return thumbnail;
    }

    public PackagePart GetThumbnailPart(PackagePart parent)
    {
        PackageRelationship relationship = null;
        PackagePart part = null;
        PackageRelationshipCollection relationshipsByType = null;
        if (parent != null)
        {
            relationshipsByType = parent.GetRelationshipsByType(DwfxMarkup.ThumbnailRelationshipName);
        }
        else
        {
            relationshipsByType = this.metroPackage.GetRelationshipsByType(DwfxMarkup.ThumbnailRelationshipName);
        }
        foreach (PackageRelationship relationship2 in relationshipsByType)
        {
            if (relationship != null)
            {
                throw new InvalidDataException(SR.Get("ReachPackaging_MoreThanOneThumbnailPart"));
            }
            relationship = relationship2;
        }
        if (relationship != null)
        {
            part = this.metroPackage.GetPart(PackUriHelper.ResolvePartUri(relationship.SourceUri, relationship.TargetUri));
        }
        return part;
    }

    public void Dispose()
    {
        this.metroPackage.Close();
        this.metroPackage = null;
    }

    public PackagePart StartingPart
    {
      get
      {
        PackagePart dwfxDocumentStartingPart = null;
        if (this.metroPackage != null)
        {
          dwfxDocumentStartingPart = GetDwfxDocumentStartingPart(this.metroPackage);
        }

        return dwfxDocumentStartingPart;
      }
    }

    internal static PackagePart GetDwfxDocumentStartingPart(Package package)
    {
      PackageRelationship relationship = null;
      PackagePart part = null;
      foreach (PackageRelationship relationship2 in package.GetRelationshipsByType(DwfxMarkup.ReachDwfxPackageStartingPartRelationshipType))
      {
        if (relationship != null)
        {
          throw new InvalidDataException(SR.Get("ReachPackaging_MoreThanOneStartingParts"));
        }

        relationship = relationship2;
      }

      if (relationship != null)
      {
        Uri partUri = PackUriHelper.ResolvePartUri(relationship.SourceUri, relationship.TargetUri);
        if (package.PartExists(partUri))
        {
          part = package.GetPart(partUri);
        }
      }

      return part;
    }

      public DwfxDocument DwfxDocument
    {
      get
      {
        return this.dwfxDocument;
      }
      set
      {
        this.dwfxDocument = value;
      }
    }

    public static bool SupportedImageType(ContentType imageContentType)
    {
      return true;
    }
  }
}
